@@ -1,24 +1,22 @@
 """Adapter for google appengine source.
 
 :organization: Logilab
-:copyright: 2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2008-2009 LOGILAB S.A. (Paris, FRANCE), license is LGPL v2.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: GNU Lesser General Public License, v2.1 - http://www.gnu.org/licenses
 """
 __docformat__ = "restructuredtext en"
 
-from logilab.common.decorators import cached, clear_cache
-
-from cubicweb import AuthenticationError, UnknownEid, server
+from cubicweb import AuthenticationError, UnknownEid
 from cubicweb.server.sources import AbstractSource, ConnectionWrapper
 from cubicweb.server.pool import SingleOperation
 from cubicweb.server.utils import crypt_password
 from cubicweb.goa.dbinit import set_user_groups
 from cubicweb.goa.rqlinterpreter import RQLInterpreter
 
-from google.appengine.api.datastore import Key, Entity, Get, Put, Delete
-from google.appengine.api.datastore import Query
+from google.appengine.api.datastore import Key, Entity, Put, Delete
 from google.appengine.api import datastore_errors, users
-    
+
 def _init_groups(guser, euser):
     # set default groups
     if guser is None:
@@ -39,11 +37,11 @@ def _clear_related_cache(session, gaesubject, rtype, gaeobject):
             pass
         else:
             entity.clear_related_cache(rtype, role)
-    if gaesubject.kind() == 'EUser':
+    if gaesubject.kind() == 'CWUser':
         for asession in session.repo._sessions.itervalues():
             if asession.user.eid == subject:
                 asession.user.clear_related_cache(rtype, 'subject')
-    if gaeobject.kind() == 'EUser':
+    if gaeobject.kind() == 'CWUser':
         for asession in session.repo._sessions.itervalues():
             if asession.user.eid == object:
                 asession.user.clear_related_cache(rtype, 'object')
@@ -85,13 +83,13 @@ def _rdel(session, gaeentity, targetkey, relation, card):
             gaeentity[relation] = related or None
     _mark_modified(session, gaeentity)
 
-    
+
 class DatastorePutOp(SingleOperation):
     """delayed put of entities to have less datastore write api calls
 
     * save all modified entities at precommit (should be the first operation
       processed, hence the 0 returned by insert_index())
-      
+
     * in case others precommit operations modify some entities, resave modified
       entities at commit. This suppose that no db changes will occurs during
       commit event but it should be the case.
@@ -106,10 +104,10 @@ class DatastorePutOp(SingleOperation):
             assert not eid in pending
             Put(gaeentity)
         modified.clear()
-        
+
     def commit_event(self):
         self._put_entities()
-        
+
     def precommit_event(self):
         self._put_entities()
 
@@ -117,12 +115,12 @@ class DatastorePutOp(SingleOperation):
 class GAESource(AbstractSource):
     """adapter for a system source on top of google appengine datastore"""
 
-    passwd_rql = "Any P WHERE X is EUser, X login %(login)s, X upassword P"
-    auth_rql = "Any X WHERE X is EUser, X login %(login)s, X upassword %(pwd)s"
-    _sols = ({'X': 'EUser', 'P': 'Password'},)
-    
+    passwd_rql = "Any P WHERE X is CWUser, X login %(login)s, X upassword P"
+    auth_rql = "Any X WHERE X is CWUser, X login %(login)s, X upassword %(pwd)s"
+    _sols = ({'X': 'CWUser', 'P': 'Password'},)
+
     options = ()
-    
+
     def __init__(self, repo, appschema, source_config, *args, **kwargs):
         AbstractSource.__init__(self, repo, appschema, source_config,
                                 *args, **kwargs)
@@ -131,11 +129,11 @@ class GAESource(AbstractSource):
             self.authenticate = self.authenticate_gauth
         else:
             self.authenticate = self.authenticate_local
-            
+
     def reset_caches(self):
         """method called during test to reset potential source caches"""
         pass
-    
+
     def init_creating(self):
         pass
 
@@ -147,7 +145,7 @@ class GAESource(AbstractSource):
 
     def get_connection(self):
         return ConnectionWrapper()
-    
+
     # ISource interface #######################################################
 
     def compile_rql(self, rql):
@@ -155,22 +153,22 @@ class GAESource(AbstractSource):
         rqlst.restricted_vars = ()
         rqlst.children[0].solutions = self._sols
         return rqlst
-    
+
     def set_schema(self, schema):
         """set the application'schema"""
         self.interpreter = RQLInterpreter(schema)
         self.schema = schema
-        if 'EUser' in schema and not self.repo.config['use-google-auth']:
+        if 'CWUser' in schema and not self.repo.config['use-google-auth']:
             # rql syntax trees used to authenticate users
             self._passwd_rqlst = self.compile_rql(self.passwd_rql)
             self._auth_rqlst = self.compile_rql(self.auth_rql)
-                
+
     def support_entity(self, etype, write=False):
         """return true if the given entity's type is handled by this adapter
         if write is true, return true only if it's a RW support
         """
         return True
-    
+
     def support_relation(self, rtype, write=False):
         """return true if the given relation's type is handled by this adapter
         if write is true, return true only if it's a RW support
@@ -188,7 +186,7 @@ class GAESource(AbstractSource):
         # XXX http://code.google.com/appengine/docs/users/userobjects.html
         # use a reference property to automatically work with email address
         # changes after the propagation feature is implemented
-        key = Key.from_path('EUser', 'key_' + login, parent=None)
+        key = Key.from_path('CWUser', 'key_' + login, parent=None)
         try:
             euser = session.datastore_get(key)
             # XXX fix user. Required until we find a better way to fix broken records
@@ -198,14 +196,14 @@ class GAESource(AbstractSource):
             return str(key)
         except datastore_errors.EntityNotFoundError:
             # create a record for this user
-            euser = Entity('EUser', name='key_' + login)
+            euser = Entity('CWUser', name='key_' + login)
             euser['s_login'] = login
             _init_groups(guser, euser)
             Put(euser)
             return str(euser.key())
-        
+
     def authenticate_local(self, session, login, password):
-        """return EUser eid for the given login/password if this account is
+        """return CWUser eid for the given login/password if this account is
         defined in this source, else raise `AuthenticationError`
 
         two queries are needed since passwords are stored crypted, so we have
@@ -227,8 +225,8 @@ class GAESource(AbstractSource):
             return rset[0][0]
         except IndexError:
             raise AuthenticationError('bad password')
-    
-    def syntax_tree_search(self, session, union, args=None, cachekey=None, 
+
+    def syntax_tree_search(self, session, union, args=None, cachekey=None,
                            varmap=None):
         """return result from this source for a rql query (actually from a rql
         syntax tree and a solution dictionary mapping each used variable to a
@@ -238,25 +236,25 @@ class GAESource(AbstractSource):
         results, description = self.interpreter.interpret(union, args,
                                                           session.datastore_get)
         return results # XXX description
-                
+
     def flying_insert(self, table, session, union, args=None, varmap=None):
         raise NotImplementedError
-    
+
     def add_entity(self, session, entity):
         """add a new entity to the source"""
         # do not delay add_entity as other modifications, new created entity
         # needs an eid
         entity.put()
-        
+
     def update_entity(self, session, entity):
         """replace an entity in the source"""
         gaeentity = entity.to_gae_model()
         _mark_modified(session, entity.to_gae_model())
-        if gaeentity.kind() == 'EUser':
+        if gaeentity.kind() == 'CWUser':
             for asession in self.repo._sessions.itervalues():
                 if asession.user.eid == entity.eid:
                     asession.user.update(dict(gaeentity))
-                
+
     def delete_entity(self, session, etype, eid):
         """delete an entity from the source"""
         # do not delay delete_entity as other modifications to ensure
@@ -273,7 +271,7 @@ class GAESource(AbstractSource):
         _radd(session, gaesubj, gaeobj.key(), 's_' + rtype, cards[0])
         _radd(session, gaeobj, gaesubj.key(), 'o_' + rtype, cards[1])
         _clear_related_cache(session, gaesubj, rtype, gaeobj)
-            
+
     def delete_relation(self, session, subject, rtype, object):
         """delete a relation from the source"""
         gaesubj, gaeobj, cards = _rinfo(session, subject, rtype, object)
@@ -283,7 +281,7 @@ class GAESource(AbstractSource):
         if not object in pending:
             _rdel(session, gaeobj, gaesubj.key(), 'o_' + rtype, cards[1])
         _clear_related_cache(session, gaesubj, rtype, gaeobj)
-        
+
     # system source interface #################################################
 
     def eid_type_source(self, session, eid):
@@ -293,7 +291,7 @@ class GAESource(AbstractSource):
         except datastore_errors.BadKeyError:
             raise UnknownEid(eid)
         return key.kind(), 'system', None
-    
+
     def create_eid(self, session):
         return None # let the datastore generating key
 
@@ -306,15 +304,14 @@ class GAESource(AbstractSource):
         record from the entities table to the deleted_entities table
         """
         pass
-        
+
     def fti_unindex_entity(self, session, eid):
         """remove text content for entity with the given eid from the full text
         index
         """
         pass
-        
+
     def fti_index_entity(self, session, entity):
         """add text content of a created/modified entity to the full text index
         """
         pass
-
